// by Gynvael Coldwind
// http://gynvael.coldwind.pl
// email: gynvael@coldwind.pl
// Consider this to be BSD style
// For licensing details contact the author.
//
// Compile: g++ sincos.exe -lgdi32
//
#include <windows.h>
#include <math.h>
#include <time.h>
#include <cstdio>
#include <cstring>
#include <cstdlib>

#define PI 3.14

#define WIDTH 1024
#define HEIGHT 1024

#pragma pack(push, 1)
struct RGB
{
  unsigned char r,g,b;
} image[WIDTH*HEIGHT], img2[WIDTH*HEIGHT], *pal, ultra[6400*6400];
#pragma pack(pop)

bool keys[1024];
bool FirstRun;

using namespace std;

char *FileGetContent(const char *filename, int *sz)
{
  char *data;
  FILE *f;
  int size;

  f = fopen(filename, "rb");
  if(!f)
    return NULL;

  fseek(f, 0, SEEK_END);
  size = (int)ftell(f);
  fseek(f, 0, SEEK_SET);

  data = new char[size];

  fread(data, 1, size, f);
  fclose(f);

  *sz = size;

  return data;  
}

bool FileSetContent(const char *filename, char *data, int sz)
{
  FILE *f;
  f = fopen(filename, "wb");
  if(f)
  {
    fwrite(data, 1, sz, f);
    fclose(f);
  }
  else
  {
    return false;
  }

  return true;
}




/* definicje
 */
LRESULT CALLBACK WndProc( HWND, UINT, WPARAM, LPARAM );

double gw;

void calc()
{
  double x = 0, y = 0;
  int ix, iy;
  int i;
  double rt = 0;
  bool chg = false;

  if(FirstRun) { chg = true; FirstRun = false; }

  static double ratio = 0.1f;
 

  static double A=-1.330000,C=0.120000,E= 0.910000,G= 2.000000;
  static double vx = 200, vy = 0;
  static double zx = 17, zy = 106;
  static int iter = 1000000, citer = 5;

  static double delta = 0.1f;
  static double max_x, max_y, min_x, min_y;

  static bool zoomed = true;

  if(keys[(unsigned)'Q']) { A += delta; chg = true; }
  if(keys[(unsigned)'A']) { A -= delta; chg = true; } // dzieki ;*
  if(keys[(unsigned)'W']) { C += delta; chg = true; }
  if(keys[(unsigned)'S']) { C -= delta; chg = true; }
  if(keys[(unsigned)'E']) { E += delta; chg = true; }
  if(keys[(unsigned)'D']) { E -= delta; chg = true; }
  if(keys[(unsigned)'R']) { G += delta; chg = true; }
  if(keys[(unsigned)'F']) { G -= delta; chg = true; }

  if(keys[(unsigned)'3']) { delta *= 10.0f; chg = true; }
  if(keys[(unsigned)'4']) { delta /= 10.0f; chg = true; }

  if(keys[(unsigned)'0']) { keys[(unsigned)'0'] = false; chg = true; } // refresh
  if(!zoomed || keys[(unsigned)'9']) 
  {
    vx = -min_x;
    vy = -min_y;
    zx = (double)(WIDTH - 100) / (max_x - min_x);
    zy = (double)(HEIGHT - 100) / (max_y - min_y);

    keys[(unsigned)'9'] = false;
    chg = true; 
    zoomed = true;
  } // fit to window

  if(keys[VK_UP]) { vy += 100; chg = true; }
  if(keys[VK_DOWN]) { vy -= 100; chg = true; }

  if(keys[VK_LEFT]) { vx += 100; chg = true; }
  if(keys[VK_RIGHT]) { vx -= 100; chg = true; }

  if(keys[(unsigned)'Z']) { zx += 1; chg = true; }
  if(keys[(unsigned)'X']) { zx -= 1; chg = true; }

  if(keys[(unsigned)'C']) { zy += 1; chg = true; }
  if(keys[(unsigned)'V']) { zy -= 1; chg = true; }

  if(keys[(unsigned)'B']) { iter += 1000000; chg = true; }
  if(keys[(unsigned)'N']) { iter -= 1000000; chg = true; }

  if(keys[(unsigned)'1']) { citer += 1; chg = true; }
  if(keys[(unsigned)'2']) { citer -= 1; chg = true; }

  if(keys[(unsigned)'5']) { ratio += 0.01; chg = true; }
  if(keys[(unsigned)'6']) { ratio -= 0.01; chg = true; }

  if(keys[(unsigned)'M'])
  {
    keys[(unsigned)'M'] = false;
    char name[1234];
    sprintf(name, "dump_%x.raw", (unsigned int)time(0));
    FILE *f = fopen("fajne.txt", "a");
    fprintf(f, "%s %f %f %f %f\n(%f,%f %f,%f) [%i %i] %f %f\n",name,A,C,E,G, vx, vy, zx, zy, iter, citer, delta, ratio);
    fclose(f);

    f = fopen(name, "wb");
    fwrite(image, 1, WIDTH*HEIGHT*3, f);
    fclose(f);
    printf("saved as %s\n", name);
   }


  if(!chg) return;

  printf("%f %f %f %f\n(%f,%f %f,%f) [%i %i] %f %f\n", A,C,E,G, vx, vy, zx, zy, iter, citer, delta, ratio);

  memset(image, 0, sizeof(image));


  double oldmax_x = max_x;
  double oldmax_y = max_y;
  double oldmin_x = min_x;
  double oldmin_y = min_y;

  max_x = -1000000.0;
  max_y = -1000000.0;
  min_x =  1000000.0;
  min_y =  1000000.0;

  for(i = 0; i < iter; i++)
  {
    rt += ratio;
    x += sin(rt*A) * cos(rt*C);
    y += sin(rt*E) * cos(rt*G);

    if(x > max_x) max_x = x;
    if(y > max_y) max_y = y;
    if(x < min_x) min_x = x;
    if(y < min_y) min_y = y;

    //printf("%f %f\n", x, y);
    ix = (int)((x + vx) * zx + 50);
    iy = (int)((y + vy) * zy + 50);

    if(ix >= 1 && ix < WIDTH-1 &&
       iy >= 1 && iy < HEIGHT-1)
    {
#define ADD_PIXEL(a,b,c) if(image[(a)+(b)*WIDTH].g + (c) < 255) image[(a)+(b)*WIDTH].g+=(c);
      ADD_PIXEL(ix, iy, citer);
    /*ADD_PIXEL(ix-1, iy-1, citer/3);
      ADD_PIXEL(ix-0, iy-1, citer/2);
      ADD_PIXEL(ix+1, iy-1, citer/3);

      ADD_PIXEL(ix-1, iy-0, citer/2);
      ADD_PIXEL(ix+1, iy-0, citer/2);

      ADD_PIXEL(ix-1, iy+1, citer/3);
      ADD_PIXEL(ix-0, iy+1, citer/2);
      ADD_PIXEL(ix+1, iy+1, citer/3);*/



#undef ADD_PIXEL


    }
    else
    {
     // printf("%i %i\n", ix, iy);
    }

  } 

  if(oldmin_y != min_y ||
     oldmax_y != max_y ||
     oldmin_x != min_x ||
     oldmax_x != max_x)
  {
    zoomed = false;
  }

    printf("x = < %f, %f > ... y = < %f, %f >\n", min_x, max_x, min_y, max_y);

 /* for(j = 0; j < 640; j++)
    for(i = 0; i < 640; i++)
      image[i+j*640].g = ultra[i*10+(j*10)*6400].g;*/
}


/* funkcja glowna
 */
int WINAPI 
WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
         LPSTR lpCmdLine, int nCmdShow )
{
  (void)hPrevInstance;
  (void)lpCmdLine;
  WNDCLASSEX wc;
  HWND hwnd;
  MSG Msg;

  FirstRun = true;

  int size;
  pal = (RGB*)FileGetContent("paleta.raw", &size);

  wc.cbSize         = sizeof( WNDCLASSEX );
  wc.style          = 0;
  wc.lpfnWndProc    = WndProc;
  wc.cbClsExtra     = 0;
  wc.cbWndExtra     = 0;
  wc.hInstance      = hInstance;
  wc.hIcon          = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor        = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground  = (HBRUSH)(COLOR_WINDOW);
  wc.lpszMenuName   = NULL;
  wc.lpszClassName  = "MyWindowsSth";
  wc.hIconSm        = LoadIcon(NULL, IDI_APPLICATION);
  
  if( !RegisterClassEx( &wc ) )
  {
    MessageBox( NULL, "Window Registration Failed!", "Error!",
                MB_ICONEXCLAMATION | MB_OK );
    return 0;
  }

 

  hwnd = CreateWindowEx(
   0,
   "MyWindowsSth",
   "SIN*COS",
   WS_OVERLAPPEDWINDOW,
   150, 50,
   WIDTH, HEIGHT,
   NULL, NULL, hInstance, NULL);

  if( !hwnd )
  {
    MessageBox( NULL, "Window Creation Failed!", "Error!",
                MB_ICONEXCLAMATION | MB_OK );
    return 0;
  }

  ShowWindow( hwnd, nCmdShow );
  UpdateWindow( hwnd );

  SetTimer(hwnd, 0, 20, NULL); 

  while( GetMessage( &Msg, NULL, 0, 0 ) )
  {
    TranslateMessage( &Msg );
    DispatchMessage( &Msg );
  }

  return Msg.wParam;
}

static BITMAPINFOHEADER bmhdr =
{
  sizeof(BITMAPINFOHEADER),
  WIDTH,
  -HEIGHT,
  1,
  24,
  BI_RGB,
  0,
  0,
  0,
  0,
  0
};


/* callback WndProc
 */
LRESULT CALLBACK 
WndProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
  HDC hdc;
  PAINTSTRUCT ps;

  switch( msg )
  {
    case WM_CLOSE:
    DestroyWindow( hwnd );
    break;

    case WM_TIMER:
    calc();
    InvalidateRect(hwnd, NULL, 0);
    UpdateWindow(hwnd);
    break;

    case WM_PAINT:
    hdc = BeginPaint(hwnd, &ps);
    StretchDIBits(hdc, 0, 0, WIDTH, HEIGHT, 0, 0, WIDTH, HEIGHT, image, (BITMAPINFO*)&bmhdr, DIB_RGB_COLORS, SRCCOPY);
    EndPaint(hwnd, &ps);
    break;

    case WM_KEYDOWN:
      keys[wParam] = true;
    break;

    case WM_KEYUP:
      keys[wParam] = false;
    break;

    case WM_DESTROY:
    PostQuitMessage( 0 );
    break;

    default:
    return DefWindowProc( hwnd, msg, wParam, lParam );
  }
  return 0;
}



